if not _G.InfamySetting then

	_G.InfamySetting = {}

	InfamySetting.Path = ""
	InfamySetting.SavePath = SavePath .. "goonmod_options.txt"

end
-- Infamy Outliner
_G.InfamySetting.InfamyOutliner = _G.InfamySetting.InfamyOutliner or {}
local InfamyOutliner = _G.InfamySetting.InfamyOutliner
InfamyOutliner.SaveFile = "goonmod_infamy_outline.txt"
InfamyOutliner.MenuFile = ModPath .. "infamy_outliner_menu.txt"
InfamyOutliner.Outline = {}

_G.InfamySetting.Options = _G.InfamySetting.Options or {}
local Options = InfamySetting.Options

local mt = getmetatable( Options )
if mt == nil then
	mt = {}
	mt.__func = {}
	mt.__data = {}
	setmetatable( Options, mt )
end

function mt.__index(tbl, key)
	return mt.__data[key] or mt.__func[key]
end

function mt.__newindex(tbl, key, value)
	mt.__data[key] = value
end

local function OptionsSave( self, file_path )

	file_path = file_path or InfamySetting.SavePath

	local file = io.open(file_path, "w+")
	if file then

		file:write( json.encode(mt.__data) )
		file:close()

	else
		log("[Error] Could not save InfamySetting options!")
	end

end
rawset( mt.__func, "Save", OptionsSave )

local function OptionsLoad( self, file_path )

	file_path = file_path or InfamySetting.SavePath

	local file = io.open( file_path, "r" )
	if file then

		local file_contents = file:read("*all")
		local data = json.decode( file_contents )
		file:close()

		if data then
			mt.__data = data
		end

	else
		log("[Warning] Could not load file '" .. file_path .. "', no data loaded...")
	end

end
rawset( mt.__func, "Load", OptionsLoad )

Options:Load()

-- Options
InfamySetting.Options.InfamyOutliner 				= InfamySetting.Options.InfamyOutliner or {}
InfamySetting.Options.InfamyOutliner.Enabled 		= InfamySetting.Options.InfamyOutliner.Enabled
InfamySetting.Options.InfamyOutliner.RH 				= InfamySetting.Options.InfamyOutliner.RH or 0.8
InfamySetting.Options.InfamyOutliner.GS 				= InfamySetting.Options.InfamyOutliner.GS or 1.0
InfamySetting.Options.InfamyOutliner.BV 				= InfamySetting.Options.InfamyOutliner.BV or 0.4
InfamySetting.Options.InfamyOutliner.UseHSV 			= InfamySetting.Options.InfamyOutliner.UseHSV
if InfamySetting.Options.InfamyOutliner.Enabled == nil then
	InfamySetting.Options.InfamyOutliner.Enabled = true
end
if InfamySetting.Options.InfamyOutliner.UseHSV == nil then
	InfamySetting.Options.InfamyOutliner.UseHSV = true
end

-- Color
InfamyOutliner.Color = ColorHSVRGB:new( InfamySetting.Options.InfamyOutliner, Color.purple:with_alpha(0) )

function InfamyOutliner:IsEnabled()
	return InfamySetting.Options.InfamyOutliner.Enabled or false
end

function InfamyOutliner:LoadInfamySkillData()

	local path = SavePath .. InfamyOutliner.SaveFile
	local file = io.open( path, "r" )
	if file then
		local data = file:read("*all")
		file:close()
		if not string.is_nil_or_empty( data ) then
			InfamyOutliner.Outline = json.decode( data )
		end
	end

end

function InfamyOutliner:SaveInfamySkillData()
	
	local tree_data = managers.skilltree._global.skill_switches
	InfamyOutliner.Outline = tree_data
	tree_data = json.encode( InfamyOutliner.Outline )

	local path = SavePath .. InfamyOutliner.SaveFile
	local file = io.open( path, "w+" )
	if file then
		file:write( tostring(tree_data) )
		file:close()
	else
		log("[Error] Could not save infamy skill tree data!")
	end

end

function InfamyOutliner:ClearInfamySkillData()

	local path = SavePath .. InfamyOutliner.SaveFile
	local success, remove_error = os.remove( path )
	if success then
		InfamyOutliner.Outline = {}
	else
		log("Could not clear infamy outliner skill data: " .. tostring(remove_error))
	end

end

function InfamyOutliner:UpdateSkillItem( item, skill_id, infamy_outline, infamy_ace )

	if not InfamyOutliner:IsEnabled() then
		return
	end

	if not infamy_outline then
		infamy_outline = item._infamy_outline
		if not infamy_outline then
			return
		end
	end
	if not infamy_ace then
		infamy_ace = item._infamy_ace
		if not infamy_ace then
			return
		end
	end

	local skill_set = managers.skilltree:get_selected_skill_switch()
	local tree = managers.skilltree._global.skill_switches[ skill_set ]
	local outline = InfamyOutliner.Outline[ skill_set ]

	if not outline or not tree.skills[skill_id] or not outline.skills[skill_id] then
		return
	end

	local tree_unlocked = tree.skills[skill_id].unlocked
	local outline_unlocked = outline.skills[skill_id].unlocked

	if self:IsSkillIDSkillTree( skill_id ) and tree_unlocked > 0 then
		infamy_outline:set_alpha( 0 )
		infamy_ace:set_alpha( 0 )
		return
	end

	if tree_unlocked and tree_unlocked >= outline_unlocked then
		infamy_outline:set_alpha( 0 )
		infamy_ace:set_alpha( 0 )
		return
	end

	if outline_unlocked then
		infamy_outline:set_alpha( outline_unlocked > 0 and 1 or 0 )
		infamy_ace:set_alpha( outline_unlocked > 1 and 1 or 0 )
	end

end

function InfamyOutliner:IsSkillIDSkillTree( id )
	for k, v in pairs( tweak_data.skilltree.trees ) do
		if v.skill == id then
			return true
		end
	end
	return false
end

function InfamyOutliner:GetColor(alpha)
	return InfamyOutliner.Color:GetColor( alpha )
end

function InfamyOutliner:ShowPreviewMenuItem()

	if not managers.menu_component then
		return
	end

	local ws = managers.menu_component._ws
	self._panel = ws:panel():panel()

	local w, h = self._panel:w() * 0.35, 48
	self._color_rect = self._panel:rect({
		w = w,
		h = h,
		color = Color.red,
		blend_mode = "add",
		layer = tweak_data.gui.MOUSE_LAYER - 50,
	})
	self._color_rect:set_right( self._panel:right() )
	self._color_rect:set_top( self._panel:h() * 0.265 )

	self:UpdatePreview()

end

function InfamyOutliner:DestroyPreviewMenuItem()

	if alive(self._panel) then

		self._panel:remove( self._color_rect )
		self._panel:remove( self._panel )

		self._color_rect = nil
		self._panel = nil

	end

end

function InfamyOutliner:UpdatePreview( t )
	if not alive(self._panel) or not alive(self._color_rect) or not InfamyOutliner.Color then
		return
	end
	self._color_rect:set_color( InfamyOutliner.Color:GetColor() )
end

-- Menu
Hooks:Add("MenuManagerInitialize", "MenuManagerInitialize_", function( menu_manager )

	InfamyOutliner:LoadInfamySkillData()

	-- Callbacks
	MenuCallbackHandler.ClosedGoonModOptions = function(this)
		InfamySetting.Options:Save()
	end
	
	MenuCallbackHandler.InfamyOutlinerChangedFocus = function( node, focus )
		if focus then
			InfamyOutliner:ShowPreviewMenuItem()
		else
			InfamyOutliner:DestroyPreviewMenuItem()
		end
	end

	MenuCallbackHandler.InfamyOutlinerToggleEnabled = function( this, item )
		InfamySetting.Options.InfamyOutliner.Enabled = item:value() == "on" and true or false
		InfamyOutliner:UpdatePreview()
	end

	MenuCallbackHandler.InfamyOutlinerSetUseHSV = function( this, item )
		InfamySetting.Options.InfamyOutliner.UseHSV = item:value() == "on" and true or false
		InfamyOutliner:UpdatePreview()
	end

	MenuCallbackHandler.InfamyOutlinerSetRedHue = function( this, item )
		InfamySetting.Options.InfamyOutliner.RH = tonumber( item:value() )
		InfamyOutliner:UpdatePreview()
	end

	MenuCallbackHandler.InfamyOutlinerSetGreenSaturation = function( this, item )
		InfamySetting.Options.InfamyOutliner.GS = tonumber( item:value() )
		InfamyOutliner:UpdatePreview()
	end

	MenuCallbackHandler.InfamyOutlinerSetBlueValue = function( this, item )
		InfamySetting.Options.InfamyOutliner.BV = tonumber( item:value() )
		InfamyOutliner:UpdatePreview()
	end

	MenuCallbackHandler.InfamyOutlinerClearInfamyData = function( this, item )
		InfamyOutliner:ClearInfamySkillData()
	end

	MenuHelper:LoadFromJsonFile( InfamyOutliner.MenuFile, InfamySetting.InfamyOutliner, InfamySetting.Options.InfamyOutliner )
end)

Hooks:Add("MenuCallbackHandlerPreIncreaseInfamous", "MenuCallbackHandlerPreIncreaseInfamous_", function(self)
	InfamyOutliner:SaveInfamySkillData()
end)

Hooks:Add("NewSkillTreeSkillItemPostInit", "NewSkillTreeSkillItemPostInit.", function(self, skill_id, skill_data, skill_panel, tree_panel, tree, tier, fullscreen_panel, gui)

	if not InfamyOutliner:IsEnabled() then
		return
	end

	local icon_panel = skill_panel:child("SkillIconPanel")
	local infamy_outline = icon_panel:bitmap({
		name = "infamy_outline",
		texture = "guis/textures/pd2/hot_cold_glow",
		alpha = 0,
		color = InfamyOutliner.Color:GetColor(),
		layer = -1
	})
	infamy_outline:set_size(self._icon:w() * 2, self._icon:h() * 2)
	infamy_outline:set_blend_mode("add")
	infamy_outline:set_rotation(360)
	infamy_outline:set_center(self._icon:center())
	self._infamy_outline = infamy_outline

	local infamy_ace = icon_panel:bitmap({
		name = "infamy_outline_ace",
		texture = "guis/textures/pd2/skilltree/ace",
		alpha = 0,
		color = InfamyOutliner.Color:GetColor(),
		layer = -1
	})
	infamy_ace:set_size(self._icon:w() * 2, self._icon:h() * 2)
	infamy_ace:set_blend_mode("add")
	infamy_ace:set_rotation(360)
	infamy_ace:set_center(self._icon:center())
	self._infamy_ace = infamy_ace

	if not managers.skilltree or not InfamyOutliner.Outline then
		return
	end

	InfamyOutliner:UpdateSkillItem( self, skill_id, infamy_outline, infamy_ace )

end)

Hooks:Add("SkillTreeSkillItemPostRefresh", "SkillTreeSkillItemPostRefresh_", function(self)

	if not InfamyOutliner:IsEnabled() then
		return
	end

	InfamyOutliner:UpdateSkillItem( self, self._skill_id )

end)